<?xml version="1.0" encoding="US-ASCII"?>
<?xml-stylesheet type="text/xsl" href="../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:c="http://www.CraneSoftwrights.com/ns/XMP/"
                xmlns:cl="http://www.cranesoftwrights.com/ns/XMP/"
                xmlns:saxon="http://icl.com/saxon"
                xmlns:exsl="http://exslt.org/common"
                xmlns:redirect="http://xml.apache.org/xalan/redirect"
                xmlns:xt="http://www.jclark.com/xt"
                extension-element-prefixes="redirect saxon exsl xt"
                exclude-result-prefixes="xs xsd c cl"
                version="2.0">
<xs:doc info="$Id: Crane-XMPExtract.xsl,v 1.9 2011/01/29 18:40:37 admin Exp $"
        filename="Crane-XMPFileExtract.xsl" vocabulary="DocBook">
  <xs:title>Extracting embedded files using  XMP metadata</xs:title>
  <para>
    This stylesheet locates files embedded in the input Adobe XMP metadata
    file using Crane's methodology of using clear text escaped in an XMP
    element:
  </para>
  <itemizedlist>
    <listitem>
      each file to be extracted as clear text is found in either the element:
      <blockquote>
        <programlisting><![CDATA[<c:file xmlns:c="http://www.CraneSoftwrights.com/ns/XMP/">...</c:file>]]></programlisting>
      </blockquote>
      or the element:
      <blockquote>
        <programlisting><![CDATA[<c:file xmlns:c="http://www.cranesoftwrights.com/ns/XMP/">...</c:file>]]></programlisting>
      </blockquote>
      though the prefix is irrelevant (note that Adobe Acrobat 8 appears
      to sometimes (but not always) require the lower-case domain name in the
      URI; if anyone has any ideas on why this is, please help us and let
      us know your thoughts ... thank you).
    </listitem>
    <listitem>
      when <literal>uri=</literal> is absent, write the file to standard output
      (note this attribute cannot be absent if there is more than one
      <literal>&lt;c:file></literal> element in the XMP)
    </listitem>
    <listitem>
      when <literal>uri=</literal> is present, write the file to the given
      filename
    </listitem>
  </itemizedlist>
  <section>
    <title>Obtaining XMP metadata from a PDF file</title>
  <para>
    The input to this stylesheet is the XMP metadata exported 
    or extracted from a PDF file.
  </para>
    <section>
      <title>Exporting XMP using Adobe Acrobat</title>
  <para>When using Adobe Acrobat 8.2.5 this is accomplished as follows:</para>
  <itemizedlist>
    <listitem>
      use <literal>File/Properties</literal> to open up the document
      properties
    </listitem>
    <listitem>
      open the <literal>Description</literal> tab
    </listitem>
    <listitem>
      press the <literal>Additional Metadata</literal> button to open the
      XMP dialogue box
    </listitem>
    <listitem>
      in the left pane select <literal>Advanced</literal>
    </listitem>
    <listitem>
      press the <literal>Save...</literal> button to export the information
      to that file that is input into this stylesheet
    </listitem>
  </itemizedlist>
    </section>
    <section>
      <title>Extracting XMP using <literal>Crane-PDF2XMP.py</literal></title>
  <para>
    This package includes the <literal>Crane-PDF2XMP.py</literal> program
    written in the <ulink uri="http://www.Python.org">Python language</ulink>
    for extracting the first embedded XMP packet found in a PDF document.
  </para>
  <para>
    The standard input is the PDF file and the standard output is the first
    XMP fragment found therein.
    A non-zero error code is returned if there is a problem.
    The invocation follows this simple approach:
  </para>
      <blockquote>
        <programlisting>python Crane-PDF2XMP.py &lt;j.pdf >j.xmp</programlisting>
      </blockquote>
      <para>
        The program source code has related documentation regarding finding
        XMP content of a PDF file by its signature processing instruction.
      </para>
    </section>
  </section>
  <section>
  <title>
    Example XMP
  </title>
<programlisting><![CDATA[<?xpacket begin="" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="3.1-702">
  <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
    <!-- PDFA ID -->
    <rdf:Description rdf:about="" xmlns:pdfaid="http://www.aiim.org/pdfa/ns/id/">
      <pdfaid:part>1</pdfaid:part>
      <pdfaid:conformance>B</pdfaid:conformance>
    </rdf:Description>

    <!-- XMP -->
    <rdf:Description rdf:about="" xmlns:xmp="http://ns.adobe.com/xap/1.0/">
      <xmp:CreateDate/>
      <xmp:ModifyDate/>
    </rdf:Description>
    
    <!-- PDF -->
    <rdf:Description rdf:about="" xmlns:pdf="http://ns.adobe.com/pdf/1.3/">
      <pdf:Keywords>Testing, PDF/A</pdf:Keywords>
      <pdf:Producer/>
    </rdf:Description>
    
    <!-- dublin core -->
    <rdf:Description rdf:about="" xmlns:dc="http://purl.org/dc/elements/1.1/">
      <dc:format>application/pdf</dc:format>
      <dc:title>
        <rdf:Alt>
          <rdf:li xml:lang="x-default">dc:title - should appear in pdf info.title</rdf:li>
        </rdf:Alt>
      </dc:title>
      <dc:creator>
        <rdf:Seq>
          <rdf:li>dc:creator - should appear in pdf info.author</rdf:li>
        </rdf:Seq>
      </dc:creator>
      <dc:description>
        <rdf:Alt>
          <rdf:li xml:lang="x-default">dc:description - should appear in pdf info.subject</rdf:li>
        </rdf:Alt>
      </dc:description>
    </rdf:Description>

    <!--file schema-->
    <rdf:Description rdf:about=""
                   xmlns:pdfaExtension="http://www.aiim.org/pdfa/ns/extension/"
                   xmlns:pdfaSchema="http://www.aiim.org/pdfa/ns/schema#"
                   xmlns:pdfaProperty="http://www.aiim.org/pdfa/ns/property#"
                   xmlns:pdfaType="http://www.aiim.org/pdfa/ns/type#">

      <!-- Container for all embedded extension schema descriptions -->
      <pdfaExtension:schemas>
        <rdf:Bag>
          <rdf:li rdf:parseType="Resource">  
            <!-- Optional description of schema -->
            <pdfaSchema:schema>Crane file container schema</pdfaSchema:schema>
            
            <!-- Schema namespace URI -->
            <pdfaSchema:namespaceURI>http://www.cranesoftwrights.com/ns/XMP/</pdfaSchema:namespaceURI>
            
            <!-- Preferred schema namespace prefix -->
            <pdfaSchema:prefix>c</pdfaSchema:prefix>
            
            <!-- Description of schema properties -->
            <pdfaSchema:property>
              <rdf:Seq>
                <rdf:li rdf:parseType="Resource">
                  <pdfaProperty:name>file</pdfaProperty:name>
                  <pdfaProperty:valueType>Text</pdfaProperty:valueType>
                  <pdfaProperty:category>external</pdfaProperty:category>
                  <pdfaProperty:description>Embedded file</pdfaProperty:description>
                </rdf:li>
              </rdf:Seq>
            </pdfaSchema:property>
            <pdfaSchema:valueType>
              <rdf:Seq>
                <rdf:li rdf:parseType="Resource">
                  <pdfaType:type>Text</pdfaType:type>
                  <pdfaType:description>Embedded file</pdfaType:description>
                </rdf:li>
              </rdf:Seq>
            </pdfaSchema:valueType>
          </rdf:li>
        </rdf:Bag>
      </pdfaExtension:schemas>
    </rdf:Description>
    
    <!--file content-->
    <rdf:Description rdf:about="">
      <c:file xmlns:c="http://www.cranesoftwrights.com/ns/XMP/">
        ...file content as text with any XML markup escaped...
      </c:file>
    </rdf:Description>
  </rdf:RDF>
  </x:xmpmeta>
  <?xpacket end="w"?>]]></programlisting>
    <!--
  <para>
    Note that to test PDF-A/1b compliance using Acrobat 8.1.7 do the following:
    <itemizedlist>
      <listitem>
        open the file and select <literal>Advanced/Pre-flight</literal>
      </listitem>
      <listitem>
        under the <literal>Profiles</literal> tab in "PDF/A compliance"
        select "Verify compliance with PDF/A-1b"
      </listitem>
      <listitem>
        press the <literal>Execute</literal> button
      </listitem>
    </itemizedlist>
  </para>
    -->
  </section>
</xs:doc>

<!--========================================================================-->
<xs:doc>
  <xs:title>Stylesheet invocation parameters</xs:title>
</xs:doc>

<xs:output>
  <para>
    The key to this approach is that the content of the element is a simple 
    text file, even if it contains markup characters.
  </para>
</xs:output>
<xsl:output method="text"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Extraction logic</xs:title>
</xs:doc>

<xs:template>
  <para>
    Extract all files from the XMP content.  If only one file is embedded,
    then when the URI attribute is absent the content is put to the standard 
    output.  If there is more than one file embedded, then each file needs
    to have a <literal>uri=</literal> attribute to specify the filename for
    export.
  </para>
  <para>
    The logic uses possible extensions to cover as many processors as
    possible.
  </para>
</xs:template>
<xsl:template match="/">
  <xsl:variable name="found" select="//c:file | //cl:file"/>
  <xsl:if test="count($found)=0">
    <xsl:message terminate="yes">
      <xsl:text>No embedded files found.</xsl:text>
    </xsl:message>
  </xsl:if>
  <xsl:if test="count($found)>1 and $found[not(@uri)]">
    <xsl:message terminate="yes">
      <xsl:text>When there is more than one embedded file, all </xsl:text>
      <xsl:text>embedded files must have the uri= attribute.</xsl:text>
    </xsl:message>
  </xsl:if>
  <xsl:for-each select="$found">
    <xsl:choose>
      <xsl:when test="not(@uri)"><!--then simply output to standard out-->
        <xsl:value-of select="."/>
      </xsl:when>
      <xsl:otherwise><!--accommodate the supplied uri location attribute-->
        <xsl:result-document href="{@uri}">
          <xsl:value-of select="."/>
          <xsl:fallback>
            <saxon:output href="{@uri}">
              <xsl:value-of select="."/>
              <xsl:fallback>
                <redirect:open file="{@uri}"/>
                <redirect:write file="{@uri}">
                  <xsl:value-of select="."/>
                  <xsl:fallback>
                    <exsl:document href="{@uri}">
                      <xsl:value-of select="."/>
                      <xsl:fallback>
                        <xt:document href="{@uri}">
                          <xsl:value-of select="."/>
                          <xsl:fallback>
                            <xsl:message terminate="yes"
                                     >Unsupported XSLT processor.</xsl:message>
                          </xsl:fallback>
                        </xt:document>
                      </xsl:fallback>
                    </exsl:document>
                  </xsl:fallback>
                </redirect:write>
                <redirect:close file="{@uri}"/>
              </xsl:fallback>
            </saxon:output>
          </xsl:fallback>
        </xsl:result-document>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
