<?xml version="1.0" encoding="US-ASCII"?>
<!--
CraneUBL2UNLK-common.xsl - wrapped stylesheet for Crane layouts to UNLK
                         - this stylesheet produces parts of an instance of
                           HTML suitable for viewing in a browser
                         - this stylesheet also contains common behaviours for
                           both HTML and XSL-FO; while these common behaviours
                           could have been put elsewhere and imported by this
                           module, by putting it here there are fewer
                           stylesheets deployed for HTML browsing 

 $Id: CraneUBL2UNLK-common.xsl,v 3.16 2011/02/03 15:33:33 admin Exp $
-->
<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-un-s.htm

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:un-in="http://www.CraneSoftwrights.com/ns/unlk/in" 
  xmlns:un-out="http://www.CraneSoftwrights.com/ns/unlk/out" 
  xmlns:cac="urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2"
  xmlns:cbc="urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2"
  xmlns:in="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2"
  version="2.0" exclude-result-prefixes="un-in un-out cac cbc in">

<xsl:output encoding="UTF-8" method="html"/>

<!--========================================================================-->
<!--identifying meta data-->

<xsl:template name="unlk-common-identification">
CraneUBL2UNLK-common.xsl - $Date: 2011/02/03 15:33:33 $(UTC)
</xsl:template>

<!--adding punctuation in the presence of an alternative language-->
<xsl:variable name="un-out:language"/>
<xsl:template name="un-out:check_alternate_form-title">
  <xsl:if test="$un-out:language"> - </xsl:if>
  <xsl:call-template name="un-out:alternate_form-title"/>
</xsl:template>
<xsl:template name="un-out:alternate_form-title"/>
<xsl:template name="un-out:check_alternate_revision">
  <xsl:if test="$un-out:language"> - </xsl:if>
  <xsl:call-template name="un-out:alternate_revision"/>
</xsl:template>
<xsl:template name="un-out:alternate_revision"/>

<!--========================================================================-->
<!--a typical line output with four possible orientations-->
<!--call the template when the line content has to be composed-->
<!--apply the template when the line applies to a single node-->

<xsl:template name="line" mode="line" match="*">
  <xsl:param name="content">
    <xsl:apply-templates mode="content"/>
  </xsl:param>
  <div class="crane-cell-data crane-cell-data-start">
    <xsl:copy-of select="$content"/>
  </div>
</xsl:template>

<xsl:template name="line-centre" mode="line-centre" match="*">
  <xsl:param name="content">
    <xsl:apply-templates mode="content"/>
  </xsl:param>
  <div class="crane-cell-data crane-cell-data-center">
    <xsl:copy-of select="$content"/>
  </div>
</xsl:template>

<xsl:template name="line-end" mode="line-end" match="*">
  <xsl:param name="content">
    <xsl:apply-templates mode="content"/>
  </xsl:param>
  <div class="crane-cell-data crane-cell-data-end">
    <xsl:copy-of select="$content"/>
  </div>
</xsl:template>

<xsl:template name="line-both">
  <xsl:param name="content-start" select="/.."/>
  <xsl:param name="content-end" select="/.."/>
  <div class="crane-cell-data crane-cell-data-justify">
    <xsl:copy-of select="$content-start"/>
    <xsl:if test="string($content-end)">
      <xsl:text> &#xa0; </xsl:text>
      <xsl:copy-of select="$content-end"/>
    </xsl:if>
  </div>
</xsl:template>

<!--========================================================================-->
<!--currency information-->

<!--a currency display; override this with an alternative order of items-->
<xsl:template mode="currency-complete" match="*">
  <xsl:call-template name="line-end">
    <xsl:with-param name="content">
      <xsl:apply-templates mode="currency-number-only" select="."/>
      <xsl:text>&#x200b;</xsl:text>
      <xsl:apply-templates mode="content" select="@currencyID"/>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!--a currency value; override this with an alternative value display-->
<xsl:param name="alternate-currency" select="false()"/>
<!--turning off currency formatting for diagnostics-->
<xsl:param name="no-currency-formatting" select="false()"/>
<xsl:decimal-format name="alternate-currency"
                    decimal-separator="," grouping-separator=" "/>
<!--note the bizarre significance issue addresses a binary representation of
    numbers quirk; this code mimicks the significance of the data entry-->
<xsl:template mode="currency-number-only" match="*">
  <xsl:choose>
    <xsl:when test="$no-currency-formatting or number(.)!=number(.)">
      <xsl:value-of select="."/>
    </xsl:when>
    <xsl:when test="$alternate-currency">
     <xsl:value-of select="format-number(number(.),concat( '# ##0,00',
                           substring(translate(.,'0123456789','##########'),
                                     2+string-length(substring-before(.,'.')),
                                     string-length(substring-after(.,'.'))-2)),
                                         'alternate-currency' )"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="format-number(number(.),concat( '#,##0.00',
                           substring(translate(.,'0123456789','##########'),
                                     2+string-length(substring-before(.,'.')),
                                     string-length(substring-after(.,'.'))-2))
                                        )"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--========================================================================-->
<!--party information in a modular form; current node is Party (or equiv)-->
<!--all this code is independent of the result markup implementation-->

<!--parties and address-->
<xsl:template mode="party-address" match="*">
  <xsl:param name="address" select="cac:PostalAddress"/>
  <xsl:apply-templates mode="scheme-value" 
                       select="cac:PartyIdentification/cbc:ID"/>
  <xsl:apply-templates mode="line" select="cac:PartyName/cbc:Name"/>
  <xsl:apply-templates mode="address" select="$address"/>
  <xsl:apply-templates mode="scheme-value" select="cbc:EndPointID"/>
  <xsl:apply-templates mode="scheme-value" 
                       select="cac:PartyLegalEntity/cbc:CompanyID"/>
</xsl:template>

<xsl:template mode="scheme-value" match="*">
  <xsl:call-template name="line">
    <xsl:with-param name="content">
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="@schemeID"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="."/>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!--bank information-->
<xsl:template mode="bank-line" match="*">
  <xsl:call-template name="line">
    <xsl:with-param name="content">
        <!--
             if " branch contains institution "
             then display branch only 
             else display institution and then branch
        -->
      <xsl:if test="string(cac:FinancialInstitution/cbc:Name) and
                   not(contains(cbc:Name, cac:FinancialInstitution/cbc:Name))">
        <xsl:apply-templates mode="content" 
                             select="cac:FinancialInstitution/cbc:Name"/>
        <xsl:text> </xsl:text>
      </xsl:if>
      <xsl:apply-templates mode="content" select="cbc:Name"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="inline-address"
                           select="( cac:FinancialInstitution/cac:Address |
                                   cac:Address )[last()]"/>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!--========================================================================-->
<!--address information in a modular form; current node is Address (or equi)-->

<!--inline address-->
<xsl:template mode="inline-address" match="*">
  <xsl:apply-templates mode="content" select="cbc:StreetName"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates mode="content" select="cbc:BuildingNumber"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates mode="content" select="cbc:PostalZone"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates mode="content" select="cbc:CityName"/>
  <xsl:call-template name="subentity"/>
  <xsl:call-template name="country"/>
</xsl:template>

<!--full address-->
<xsl:template mode="address" match="*">
  <xsl:call-template name="line">
    <xsl:with-param name="content">
      <xsl:apply-templates mode="content" select="cbc:StreetName"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="cbc:BuildingNumber"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates mode="line" select="cac:AddressLine/cbc:Line"/>
  <xsl:apply-templates mode="line" select="cbc:CitySubdivisionName"/>
  <xsl:call-template name="line">
    <xsl:with-param name="content">
      <!--only display the country code if other country info not present-->
      <xsl:if test="not(concat(cbc:CountrySubentityCode,
                               cbc:CountrySubentity,cac:Country/cbc:Name))">
        <xsl:apply-templates mode="content" 
                             select="cac:Country/cbc:IdentificationCode"/>
      </xsl:if>
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="cbc:PostalZone"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="cbc:CityName"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="line">
    <xsl:with-param name="content">
      <xsl:call-template name="subentity"/>
      <xsl:call-template name="country"/>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="subentity">
  <xsl:choose>
    <xsl:when test="cbc:CountrySubentityCode">
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="cbc:CountrySubentityCode"/>
    </xsl:when>
    <xsl:when test="cbc:CountrySubentity">
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="cbc:CountrySubentity"/>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="country">
  <xsl:choose>
    <xsl:when test="cac:Country/cbc:Name">
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" select="cac:Country/cbc:Name"/>
    </xsl:when>
    <xsl:when test="cac:Country/cbc:IdentificationCode">
      <xsl:text> </xsl:text>
      <xsl:apply-templates mode="content" 
                           select="cac:Country/cbc:IdentificationCode"/>
    </xsl:when>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
