<?xml version="1.0" encoding="US-ASCII"?>
<!--
CraneUBL2UNLK-print.xsl - support stylesheet for Crane layouts for the UNLK
                        - this stylesheet produces parts of an instance of
                          XSL-FO suitable for formatting to PDF or other
                          print format by using an XSL-FO rendering engine

 $Id: CraneUBL2UNLK-print.xsl,v 3.10 2011/02/02 20:46:03 admin Exp $
-->
<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-un-s.htm

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->
<!DOCTYPE xsl:stylesheet
[
<!ENTITY CraneXMPns "http://www.CraneSoftwrights.com/ns/XMP/">
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:un-in="http://www.CraneSoftwrights.com/ns/unlk/in" 
  xmlns:un-out="http://www.CraneSoftwrights.com/ns/unlk/out" 
  xmlns:cac="urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2"
  xmlns:cbc="urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2"
  xmlns:in="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2"
  xmlns="http://www.w3.org/1999/XSL/Format" 
  version="2.0" exclude-result-prefixes="un-in un-out cac cbc in">

<xsl:import href="CraneUBL2UNLK-common.xsl"/>

<xsl:variable name="rootnode" select="/"/>

<xsl:output method="xml"/>

<!--========================================================================-->
<!--a typical line output with four possible orientations for XSL-FO-->

<xsl:template name="line" mode="line" match="*">
  <xsl:param name="content">
    <xsl:apply-templates mode="content"/>
  </xsl:param>
  <block start-indent="2pt" end-indent="2pt" text-align="start">
    <xsl:copy-of select="$content"/>
  </block>
</xsl:template>

<xsl:template name="line-centre" mode="line-centre" match="*">
  <xsl:param name="content">
    <xsl:apply-templates mode="content"/>
  </xsl:param>
  <block start-indent="2pt" end-indent="2pt" text-align="center">
    <xsl:copy-of select="$content"/>
  </block>
</xsl:template>

<xsl:template name="line-end" mode="line-end" match="*">
  <xsl:param name="content">
    <xsl:apply-templates mode="content"/>
  </xsl:param>
  <block start-indent="2pt" end-indent="2pt" text-align="end">
    <xsl:copy-of select="$content"/>
  </block>
</xsl:template>

<xsl:template name="line-both">
  <xsl:param name="content-start" select="/.."/>
  <xsl:param name="content-end" select="/.."/>
  <block start-indent="2pt" end-indent="2pt" text-align-last="justify">
    <inline><xsl:copy-of select="$content-start"/></inline>
    <leader leader-length.optimum=".5em"/>
    <inline><xsl:copy-of select="$content-end"/></inline>
  </block>
</xsl:template>

<!--========================================================================-->
<!--embedded  metadata-->

<!--values settable at invocation time-->
<xsl:param name="title" select="''"/>
<xsl:param name="author" select="''"/>
<xsl:param name="subject" select="''"/>
<xsl:param name="keywords" select="''"/>
<xsl:param name="user-password" select="''"/>
<xsl:param name="owner-password" select="''"/>
<xsl:param name="omit-xmp" select="''"/>

<!--this is the hook in to content after the document element-->
<xsl:template name="un-out:startup-content"
               xmlns:ibex="http://www.xmlpdf.com/2003/ibex/Format"
               xmlns:rx="http://www.renderx.com/XSL/Extensions"
               xmlns:axf="http://www.antennahouse.com/names/XSL/Extensions">
  <!--document security-->
  <xsl:if test="$user-password or $owner-password">
    <ibex:security bits="128" deny-modify="true" deny-assembly="true">
      <xsl:if test="$user-password">
        <xsl:attribute name="user-password">
          <xsl:value-of select="$user-password"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="$owner-password">
        <xsl:attribute name="owner-password">
          <xsl:value-of select="$owner-password"/>
        </xsl:attribute>
      </xsl:if>
    </ibex:security>
    <xsl:if test="$user-password">
      <xsl:processing-instruction name="xep-pdf-userpassword">
        <xsl:value-of select="$user-password"/>
      </xsl:processing-instruction>
    </xsl:if>
    <xsl:if test="$owner-password">
      <xsl:processing-instruction name="xep-pdf-ownerpassword">
        <xsl:value-of select="$owner-password"/>
      </xsl:processing-instruction>
    </xsl:if>
    <xsl:processing-instruction name="xep-pdf-userprivileges">
      <xsl:text>print copy annotate</xsl:text>
    </xsl:processing-instruction>
  </xsl:if>

  <!--document properties-->
  <ibex:properties title="{$title}" subject="{$subject}"
                   author="{$author}" keywords="{$keywords}"/>
  <rx:meta-info>
    <rx:meta-field name="title" value="{$title}"/>
    <rx:meta-field name="author" value="{$author}"/>
    <rx:meta-field name="subject" value="{$subject}"/>
    <rx:meta-field name="keywords" value="{$keywords}"/>
    <rx:meta-field name="creator" value="{$author}"/>
  </rx:meta-info>
  <xsl:if test="$title">
    <axf:document-info name="title" value="{$title}"/>
  </xsl:if>
  <xsl:if test="$author">
    <axf:document-info name="author" value="{$author}"/>
  </xsl:if>
  <xsl:if test="$subject">
    <axf:document-info name="subject" value="{$subject}"/>
  </xsl:if>
  <xsl:if test="$keywords">
    <axf:document-info name="keywords" value="{$keywords}"/>
  </xsl:if>

  <xsl:if test="not($omit-xmp)">
    <!--XMP metadata-->
    <ibex:pdfa>
      <xsl:call-template name="create-xmp"/>
    </ibex:pdfa>
  </xsl:if>

</xsl:template>

<xsl:template name="create-xmp">
  <xmp:xmpmeta xmlns:xmp="adobe:ns:meta/" xmp:xmptk="3.1-702">
    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
      
      <!-- PDFA ID -->
      <rdf:Description rdf:about="" 
                       xmlns:pdfaid="http://www.aiim.org/pdfa/ns/id/">
        <pdfaid:part>1</pdfaid:part>
        <pdfaid:conformance>B</pdfaid:conformance>
      </rdf:Description>
      
      <!-- XMP -->
      <rdf:Description rdf:about="" xmlns:xmp="http://ns.adobe.com/xap/1.0/">
        <xmp:CreateDate/>
        <xmp:ModifyDate/>
      </rdf:Description>
      
      <!-- PDF -->
      <rdf:Description rdf:about="" xmlns:pdf="http://ns.adobe.com/pdf/1.3/">
        <pdf:Keywords><xsl:value-of select="$keywords"/></pdf:Keywords>
        <pdf:Producer/>
      </rdf:Description>
      
      <!-- dublin core -->
      <rdf:Description rdf:about="" 
                       xmlns:dc="http://purl.org/dc/elements/1.1/">
        <dc:format>application/pdf</dc:format>
        <dc:title>
          <rdf:Alt>
            <rdf:li xml:lang="x-default">
              <xsl:value-of select="$title"/>
            </rdf:li>
          </rdf:Alt>
        </dc:title>
        <dc:creator>
          <rdf:Seq>
            <rdf:li><xsl:value-of select="$author"/></rdf:li>
          </rdf:Seq>
        </dc:creator>
        <dc:description>
          <rdf:Alt>
            <rdf:li xml:lang="x-default">
              <xsl:value-of select="$subject"/>
            </rdf:li>
          </rdf:Alt>
        </dc:description>
      </rdf:Description>

      <!-- file schema -->
      <rdf:Description rdf:about=""
           xmlns:pdfaExtension="http://www.aiim.org/pdfa/ns/extension/"
           xmlns:pdfaSchema="http://www.aiim.org/pdfa/ns/schema#"
           xmlns:pdfaProperty="http://www.aiim.org/pdfa/ns/property#"
           xmlns:pdfaType="http://www.aiim.org/pdfa/ns/type#">

        <!-- Container for all embedded extension schema descriptions -->
        <pdfaExtension:schemas>
          <rdf:Bag>
            <rdf:li rdf:parseType="Resource">  
              <!-- Optional description of schema -->
             <pdfaSchema:schema>Crane file container schema</pdfaSchema:schema>
              
              <!-- Schema namespace URI -->
              <pdfaSchema:namespaceURI
             >&CraneXMPns;</pdfaSchema:namespaceURI>
              
              <!-- Preferred schema namespace prefix -->
              <pdfaSchema:prefix>c</pdfaSchema:prefix>
              
              <!-- Description of schema properties -->
              <pdfaSchema:property>
                <rdf:Seq>
                  <rdf:li rdf:parseType="Resource">
                    <pdfaProperty:name>file</pdfaProperty:name>
                    <pdfaProperty:valueType>Text</pdfaProperty:valueType>
                    <pdfaProperty:category>external</pdfaProperty:category>
                    <pdfaProperty:description
                                      >Embedded file</pdfaProperty:description>
                  </rdf:li>
                </rdf:Seq>
              </pdfaSchema:property>
              <pdfaSchema:valueType>
                <rdf:Seq>
                  <rdf:li rdf:parseType="Resource">
                    <pdfaType:type>Text</pdfaType:type>
                    <pdfaType:description>Embedded file</pdfaType:description>
                  </rdf:li>
                </rdf:Seq>
              </pdfaSchema:valueType>
            </rdf:li>
          </rdf:Bag>
        </pdfaExtension:schemas>
      </rdf:Description>
      
      <rdf:Description rdf:about="">
        <c:file xmlns:c="&CraneXMPns;">
          <xsl:variable name="content-in-clear-text">
            <xsl:apply-templates select="$rootnode" mode="clear-text"/>
          </xsl:variable>
          <xsl:copy-of select="$content-in-clear-text"/>
<!--          <!-\-report information obtained for diagnostic purposes-\->
          <xsl:message>
<xsl:variable name="duri" select="document-uri(/)"/>
<xsl:text/>***document-uri(/): <xsl:value-of select="document-uri(/)"/>***
***doc-available(document-uri(/)): <xsl:value-of select="doc-available(document-uri(/))"/>***
***$duri: <xsl:value-of select="$duri"/>***
***doc-available($duri): <xsl:value-of select="doc-available($duri)"/>***
<xsl:text/>
             <xsl:text>******** 200:
</xsl:text>
              <xsl:value-of select="substring($content-in-clear-text,1,200)"/>
            <xsl:text>********
</xsl:text>
<xsl:if test="ends-with(document-uri(/),'.xml')">
<xsl:variable name="duri" select="replace(document-uri(/),'/[^/]+$','')"/>
<xsl:text/>***$duri: <xsl:value-of select="$duri"/>***
***doc-available($duri): <xsl:value-of select="doc-available($duri)"/>***
<xsl:text/>
             <xsl:text>******** 200:
</xsl:text>
              <xsl:value-of select="substring(unparsed-text($duri),1,200)"/>
            <xsl:text>********
</xsl:text>
</xsl:if>
          </xsl:message>-->
        </c:file>
      </rdf:Description>
    </rdf:RDF>
  </xmp:xmpmeta>
</xsl:template>

<!--reconstitute the input file in clear text-->
<xsl:template mode="clear-text" match="/">
  <xsl:choose>
    <xsl:when test="not(function-available('unparsed-text-available'))">
      <!--reconstitute the file-->
      <xsl:apply-templates mode="clear-text"/>
    </xsl:when>
    <xsl:when test="not(unparsed-text-available(document-uri($rootnode))) or
                    not(doc-available(document-uri($rootnode)))">
<!--                    ends-with(document-uri($rootnode),'/')">-->
      <!--reconstitute the file (accommodating buggy response to d-uri())-->
      <xsl:apply-templates mode="clear-text"/>
    </xsl:when>
    <xsl:otherwise>
      <!--copy the file verbatim because it must be XML-->
      <xsl:value-of select="unparsed-text(document-uri($rootnode))"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--reconstitute an element in clear text-->
<xsl:template mode="clear-text" match="*">
  <xsl:text/>&lt;<xsl:value-of select="name(.)"/>
  <xsl:for-each select="namespace::*[name(.)!='xml']">
    <xsl:if test="not(../../namespace::*[name(.)=name(current()) and
                                         .=current()])">
      <xsl:text> xmlns</xsl:text>
      <xsl:if test="name(.)">:<xsl:value-of select="name(.)"/></xsl:if>
      <xsl:text/>="<xsl:value-of select="."/>"<xsl:text/>
    </xsl:if>
  </xsl:for-each>
  <xsl:apply-templates mode="clear-text" select="attribute::*"/>
  <xsl:choose>
    <xsl:when test="node()">
      <xsl:text>></xsl:text>
      <xsl:apply-templates mode="clear-text"/>
      <xsl:text/>&lt;/<xsl:value-of select="name(.)"/>&gt;<xsl:text/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>/></xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--reconstitute an attribute in clear text-->
<xsl:template mode="clear-text" match="@*">
  <xsl:text> </xsl:text>
  <xsl:value-of select="name(.)"/>="<xsl:call-template 
                                  name="escape-with-double-quote"/>"<xsl:text/>
</xsl:template>

<!--reconstitute a comment in clear text-->
<xsl:template mode="clear-text" match="comment()">
  <xsl:text/>&lt;!--<xsl:value-of select="."/>--><xsl:text/>
</xsl:template>

<!--reconstitute a processing instruction in clear text-->
<xsl:template mode="clear-text" match="processing-instruction()">
  <xsl:text/>&lt;?<xsl:value-of select="name(.)"/>
  <xsl:if test="string(.)">
    <xsl:text> </xsl:text>
    <xsl:value-of select="."/>
  </xsl:if>
  <xsl:text>?></xsl:text>
</xsl:template>

<!--reconstitute text in clear text-->
<xsl:template mode="clear-text" match="text()">
  <xsl:call-template name="escape"/>
</xsl:template>

<!--escape ampersand, less-than, greater-than and double quote-->
<xsl:template name="escape-with-double-quote">
  <xsl:param name="content">
    <xsl:call-template name="escape"/>
  </xsl:param>
  <xsl:choose>
    <xsl:when test="contains($content,'&quot;')">
      <xsl:value-of select="substring-before($content,'&quot;')"/>
      <xsl:text>&amp;quot;</xsl:text>
      <xsl:call-template name="escape-with-double-quote">
        <xsl:with-param name="content"
                        select="substring-after($content,'&quot;')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$content"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--escape ampersand, less-than and greater-than-->
<xsl:template name="escape">
  <xsl:param name="content">
    <xsl:call-template name="escape-less-than"/>
  </xsl:param>
  <xsl:choose>
    <xsl:when test="contains($content,'&gt;')">
      <xsl:value-of select="substring-before($content,'&gt;')"/>
      <xsl:text>&amp;gt;</xsl:text>
      <xsl:call-template name="escape">
        <xsl:with-param name="content"
                        select="substring-after($content,'&gt;')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$content"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="escape-less-than">
  <xsl:param name="content">
    <xsl:call-template name="escape-ampersand"/>
  </xsl:param>
  <xsl:choose>
    <xsl:when test="contains($content,'&lt;')">
      <xsl:value-of select="substring-before($content,'&lt;')"/>
      <xsl:text>&amp;lt;</xsl:text>
      <xsl:call-template name="escape-less-than">
        <xsl:with-param name="content"
                        select="substring-after($content,'&lt;')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$content"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="escape-ampersand">
  <xsl:param name="content" select="."/>
  <xsl:choose>
    <xsl:when test="contains($content,'&amp;')">
      <xsl:value-of select="substring-before($content,'&amp;')"/>
      <xsl:text>&amp;amp;</xsl:text>
      <xsl:call-template name="escape-ampersand">
        <xsl:with-param name="content"
                        select="substring-after($content,'&amp;')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$content"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
